%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Technical Note TN137
% Parameters for 'I-f startup for sensorless PMSM control'
% Copyright imperix ltd, 2021
% dev@imperix.ch
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% =================== System parameters ======================= %%

ControlFreq    = 20e3;           % Control interrupt frequency [Hz]
ControlPeriod  = 1/ControlFreq;  % Control interrupt period [s]
SamplingPeriod = ControlPeriod;  % Sampling period [s]

% Deadtime
dt = 1e-6;


%% =================== Plant model ======================== %%

% Encoder parameters
PPR     = 4096;          % Encoder pulses per turn [-]

% PMSM
Pmsm.p   = 3;			% Pole pairs [-]
Pmsm.Usn = 460;			% Rated voltage [V]
Pmsm.Isn = 2.43;		% Rated current [Arms]
Pmsm.fsn = 50;			% Rated frequency [Hz]
Pmsm.wsn = 2*pi*50;		% Rated angular frequency [rad/s]
Pmsm.Rs  = 3.4;			% Stator resistance [Ohm]
Pmsm.Ls  = 12.15e-3;	% Stator inductance [H]
Pmsm.Ld  = Pmsm.Ls;     % Stator inductance along d axis [H]
Pmsm.Lq  = Pmsm.Ls;     % Stator inductance along q axis [H]

Pmsm.Jm  = 2.9e-4;		% Moment of inertia [Nm^2]
Pmsm.kF  = 0.0003035;	% Friction coefficient 
Pmsm.Phi_pm = 0.25;		% Flux from permanent magnets [Nm/Apeak]

Pmsm.Temn = 3.9; 		% Nominal torque [Nm]
Pmsm.Pn   = 1.23e3;		% Nominal active power [W]

Pmsm.Is_max = 8.1;		% Peak current [A]
Pmsm.Wm_Max = 4800;		% Max mechanical speed [rpm]

% Encoder parameters
PPR     = 4096;         % Encoder pulses per turn [-]


%% ============== DC link - Voltage control =============== %%

DcLink.Rbr  = 50;		% Braking chopper resistor [Ohm]
DcLink.Vdc0 = 600;      % Initial DC bus voltage [V]


%% =============== PMSM - Current control ================= %%

% Plant model
Ipmsm.K1 = 1 / Pmsm.Rs;
Ipmsm.T1 = Pmsm.Ld / Pmsm.Rs;
Ipmsm.K2 = 1 / Pmsm.Rs;
Ipmsm.T2 = Pmsm.Lq / Pmsm.Rs;

% Small time constants
Ipmsm.Ts = ControlPeriod;           % Current control period [s]

Ipmsm.Tsens = 0;                    % Sensing delay [s]
Ipmsm.Tctrl = 2*Ipmsm.Ts;           % Control delay [s] (worst case scenario)
Ipmsm.Tpwm = ControlPeriod;         % Modulator delay [s]

Ipmsm.Ttot = Ipmsm.Tsens + Ipmsm.Tctrl + Ipmsm.Tpwm;    % Total delay [s]
Ipmsm.Kcm = 1;

% Magnitude optimum
Ipmsm.Tn_d = Ipmsm.T1;
Ipmsm.Ti_d = 2 * Ipmsm.Kcm * Ipmsm.K1 * Ipmsm.Ttot;
Ipmsm.Tn_q = Ipmsm.T2;
Ipmsm.Ti_q = 2 * Ipmsm.Kcm * Ipmsm.K2 * Ipmsm.Ttot;

Ipmsm.Kp_d = Ipmsm.Tn_d / Ipmsm.Ti_d;
Ipmsm.Ki_d = 1 / Ipmsm.Ti_d;
Ipmsm.Kp_q = Ipmsm.Tn_q / Ipmsm.Ti_q;
Ipmsm.Ki_q = 1 / Ipmsm.Ti_q;


%% =============== PMSM - Sensorless ====================== %%

Sensorless.Ts = ControlPeriod;  % Discretization period of the sensorless algorithm

Sensorless.fc_EMF = 60;         % Back EMF filter cut-off frequency [Hz]
Sensorless.fc_wm = 10;          % Estimated speed filter cut-off frequency [Hz]

Sensorless.Ks = 20000;          % Sliding mode observer gain
Sensorless.a = 0.01;            % Sigmoid constant

QPLL.Kp = 100;                  % Proportionnal gain of the Q-PLL
QPLL.Ki = 50;                   % Integral gain of the Q-PLL

% I-f startup
Sensorless.theta_tol = 0.1;                 % Angle error tolerance [rad]
Sensorless.IqRef_tol = 0.05*Pmsm.Isn;		% Minimum current reference during startup [A]


%% =============== PMSM - Speed control =================== %%

% Plant model
Speed.T1 = Pmsm.Jm;

% Speed filter
Speed.fc = 1e3;                     % Cutoff frequency [Hz]

% Small time constants
Speed.Ratio = 100;                  % Decimation ratio of the speed control loop [s]
Speed.Ts = Speed.Ratio*Ipmsm.Ts;    % Speed controller period [s]

Speed.Tsens = 1/(2*pi*Sensorless.fc_wm) + 2/(2*pi*Sensorless.fc_EMF); % Sensing delay [s]
Speed.Tctrl = Speed.Ts;             % Control delay [s] (worst case scenario)
Speed.Tpwm = ControlPeriod;         % Modulator delay [s]

Speed.Ttot = Speed.Tsens + Speed.Tctrl + Speed.Tpwm;    % Total delay [s]
Speed.Kcm = 1;

% Symmetrical optimum
Speed.Tn = 4 * Speed.Ttot;
Speed.Ti = 8 * (Speed.Ttot^2 / Speed.T1) * Speed.Kcm;

Speed.Kp = Speed.Tn / Speed.Ti;
Speed.Ki = 1 / Speed.Ti;

% Torque saturation
Speed.Tem_sat = 1.1 * Pmsm.Temn;
